/**
 * The BlendMode enum defines how translucent cells are rendered.
 */
export declare type BlendMode = 'blend' | 'add';

/**
 * Capitalizes the first letter of the input string.
 * @param str - The original input string.
 * @returns The capitalized string.
 */
export declare function capitalize(str: string): string;

export declare class Cell {
    readonly x: number;
    readonly y: number;
    charCode: number;
    fg: Color;
    bg: Color;
    dirty: boolean;
    blocked: boolean;
    blockedSight: boolean;
    explored: boolean;
    visible: boolean;
    pathId: number;
    g: number;
    h: number;
    prev: Cell | null;
    constructor(x: number, y: number, charCode?: string | number, fg?: Color, bg?: Color);
    setCharCode(charCode: number): void;
    setForeground(fg: Color): void;
    setBackground(bg: Color): void;
    setValue(charCode: string | number, fg?: Color, bg?: Color): boolean;
    drawCell(otherCell: Cell, blendMode?: BlendMode): void;
    private blendColors;
    private clamp;
}

export declare const Chars: {
    SMILEY: number;
    INVERSE_SMILEY: number;
    HEART: number;
    DIAMOND: number;
    CLUB: number;
    SPADE: number;
    BULLET: number;
    INVERSE_BULLET: number;
    LIGHT_SHADE: number;
    MEDIUM_SHADE: number;
    DARK_SHADE: number;
    BOX_SINGLE_VERTICAL: number;
    BOX_SINGLE_VERTICAL_AND_SINGLE_LEFT: number;
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_LEFT: number;
    BOX_DOUBLE_VERTICAL: number;
    BOX_DOUBLE_DOWN_AND_DOUBLE_LEFT: number;
    BOX_DOUBLE_UP_AND_DOUBLE_LEFT: number;
    BOX_SINGLE_DOWN_AND_SINGLE_LEFT: number;
    BOX_SINGLE_UP_AND_SINGLE_RIGHT: number;
    BOX_SINGLE_HORIZONTAL_AND_SINGLE_UP: number;
    BOX_SINGLE_HORIZONTAL_AND_SINGLE_DOWN: number;
    BOX_SINGLE_VERTICAL_AND_SINGLE_RIGHT: number;
    BOX_SINGLE_HORIZONTAL: number;
    BOX_SINGLE_VERTICAL_AND_SINGLE_HORIZONTAL: number;
    BOX_DOUBLE_UP_AND_DOUBLE_RIGHT: number;
    BOX_DOUBLE_DOWN_AND_DOUBLE_RIGHT: number;
    BOX_DOUBLE_HORIZONTAL_AND_DOUBLE_UP: number;
    BOX_DOUBLE_HORIZONTAL_AND_DOUBLE_DOWN: number;
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_RIGHT: number;
    BOX_DOUBLE_HORIZONTAL: number;
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_HORIZONTAL: number;
    BOX_SINGLE_UP_AND_SINGLE_LEFT: number;
    BOX_SINGLE_DOWN_AND_SINGLE_RIGHT: number;
    BLOCK_FULL: number;
    BLOCK_BOTTOM_HALF: number;
    BLOCK_LEFT_HALF: number;
    BLOCK_RIGHT_HALF: number;
    BLOCK_TOP_HALF: number;
};

/**
 * Colodore palette.
 * Recalculated palette for the VIC II (Commodore 64) by Pepto.
 * See: https://www.pepto.de/projects/colorvic/
 * See: https://lospec.com/palette-list/colodore
 */
export declare const ColodorePalette: {
    BLACK: number;
    WHITE: number;
    RED: number;
    CYAN: number;
    VIOLET: number;
    GREEN: number;
    BLUE: number;
    YELLOW: number;
    ORANGE: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_GRAY: number;
    GRAY: number;
    LIGHT_GREEN: number;
    LIGHT_BLUE: number;
    LIGHT_GRAY: number;
};

export declare type Color = number;

/**
 * Orignal CGA palette.
 * See: https://en.wikipedia.org/wiki/Color_Graphics_Adapter
 * See: https://lospec.com/palette-list/color-graphics-adapter
 */
export declare const Colors: {
    BLACK: number;
    WHITE: number;
    LIGHT_GRAY: number;
    DARK_GRAY: number;
    YELLOW: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_RED: number;
    LIGHT_GREEN: number;
    DARK_GREEN: number;
    LIGHT_CYAN: number;
    DARK_CYAN: number;
    LIGHT_BLUE: number;
    DARK_BLUE: number;
    LIGHT_MAGENTA: number;
    DARK_MAGENTA: number;
    ORANGE: number;
};

/**
 * Commodore 64 palette.
 * See: https://www.c64-wiki.com/wiki/Color
 * See: https://lospec.com/palette-list/commodore64
 */
export declare const Commodore64Palette: {
    BLACK: number;
    WHITE: number;
    RED: number;
    CYAN: number;
    VIOLET: number;
    GREEN: number;
    BLUE: number;
    YELLOW: number;
    ORANGE: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_GRAY: number;
    GRAY: number;
    LIGHT_GREEN: number;
    LIGHT_BLUE: number;
    LIGHT_GRAY: number;
};

/**
 * Calculates path between two points using Dijkstra's algorithm.
 *
 * @param source - Starting point, must have x and y properties.
 * @param dest - Destination point, must have x and y properties.
 * @param maxDist - Maximum distance to examine.
 * @returns Array of steps if destination found; undefined otherwise.
 */
export declare function computePath(map: Console_2, source: PointLike, dest: PointLike, maxDist: number): Cell[] | undefined;

declare class Console_2 {
    readonly width: number;
    readonly height: number;
    readonly grid: Cell[][];
    originX: number;
    originY: number;
    minX: number;
    maxX: number;
    minY: number;
    maxY: number;
    radius: number;
    clip?: Rect;
    constructor(width: number, height: number, blockedFunc?: (x: number, y: number) => boolean);
    clear(): void;
    getCell(x: number, y: number): Cell | undefined;
    getCharCode(x: number, y: number): number | undefined;
    drawChar(x: number, y: number, c: string | number, fg?: Color, bg?: Color): void;
    drawString(x: number, y: number, str: string, fg?: Color, bg?: Color): void;
    drawStringLine(x: number, y: number, line: string, fg?: Color, bg?: Color): void;
    drawCenteredString(x: number, y: number, str: string, fg?: Color, bg?: Color): void;
    drawMessage(x: number, y: number, message: Message, maxWidth: number): number;
    drawHLine(x: number, y: number, width: number, c: string | number, fg?: Color, bg?: Color): void;
    drawVLine(x: number, y: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawRect(x: number, y: number, width: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawBox(x: number, y: number, width: number, height: number, topChar: number, rightChar: number, bottomChar: number, leftChar: number, topLeftChar: number, topRightChar: number, bottomRightChar: number, bottomLeftChar: number, fg?: Color, bg?: Color): void;
    drawSingleBox(x: number, y: number, width: number, height: number, fg?: Color, bg?: Color): void;
    drawDoubleBox(x: number, y: number, width: number, height: number, fg?: Color, bg?: Color): void;
    fillRect(x: number, y: number, width: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawConsole(dstX: number, dstY: number, srcConsole: Console_2, srcX: number, srcY: number, srcWidth: number, srcHeight: number, blendMode?: BlendMode): void;
    drawCell(x: number, y: number, cell: Cell, blendMode?: BlendMode): void;
    setBlocked(x: number, y: number, blocked: boolean): void;
    setBlockedSight(x: number, y: number, blockedSight: boolean): void;
    isVisible(x: number, y: number): boolean;
    isBlocked(x: number, y: number): boolean;
    isBlockedSight(x: number, y: number): boolean;
    /**
     * Compute the FOV in an octant adjacent to the Y axis
     */
    private computeOctantY;
    /**
     * Compute the FOV in an octant adjacent to the X axis
     */
    private computeOctantX;
    computeFov(originX: number, originY: number, radius: number, opt_noClear?: boolean, opt_octants?: number): void;
    /**
     * All visible tiles are marked as explored.
     */
    updateExplored(): void;
}
export { Console_2 as Console }

export declare interface CrtOptions {
    scale: number;
    blur: number;
    curvature: number;
    chroma: number;
    vignette: number;
    scanlineWidth: number;
    scanlineIntensity: number;
}

export declare const DEFAULT_FONT: Font;

export declare class DefaultDialogRenderer {
    getState(terminal: Terminal, dialog: Dialog): DialogState;
    draw(term: Terminal, dialogState: DialogState): void;
}

/**
 * Deserializes a JSON string to an object graph.
 * Handles circular references and class instances.
 * @param str - The JSON string to deserialize.
 * @returns The deserialized object graph.
 */
export declare function deserialize(str: string): unknown;

export declare abstract class Dialog {
    readonly contentsRect: Rect;
    readonly title: string;
    constructor(contentsRect: Rect, title: string);
    abstract drawContents(console: Console_2, offset: Point): void;
    abstract handleInput(terminal: Terminal, offset: Point): boolean;
}

export declare interface DialogRenderer {
    getState(terminal: Terminal, dialog: Dialog): DialogState;
    draw(term: Terminal, dialogState: DialogState): void;
}

export declare class DialogState {
    readonly dialog: Dialog;
    readonly rect: Rect;
    readonly contentsOffset: Point;
    open: boolean;
    count: number;
    buffer?: Console_2;
    constructor(dialog: Dialog, rect: Rect, contentsOffset: Point);
}

export declare function fixBoxCells(con: Console_2): void;

export declare class Font {
    readonly url: string;
    readonly charWidth: number;
    readonly charHeight: number;
    constructor(url: string, charWidth: number, charHeight: number);
}

/**
 * The FovOctants constants provide bitmasks for various directions.
 *
 * @example
 * ```
 *     \ 4 | 3 /
 *      \  |  /
 *    5  \ | /  2
 *        \|/
 *   ------+-------
 *        /|\
 *    6  / | \  1
 *      /  |  \
 *     / 7 | 0 \
 *```
 */
export declare const FovOctants: {
    OCTANT_SOUTH_SOUTHEAST: number;
    OCTANT_EAST_SOUTHEAST: number;
    OCTANT_EAST_NORTHTHEAST: number;
    OCTANT_NORTH_NORTHEAST: number;
    OCTANT_NORTH_NORTHWEST: number;
    OCTANT_WEST_NORTHEAST: number;
    OCTANT_WEST_SOUTHWEST: number;
    OCTANT_SOUTH_SOUTHWEST: number;
};

export declare const FovQuadrants: {
    QUADRANT_SOUTHEAST: number;
    QUADRANT_EAST: number;
    QUADRANT_NORTHEAST: number;
    QUADRANT_NORTH: number;
    QUADRANT_NORTHWEST: number;
    QUADRANT_WEST: number;
    QUADRANT_SOUTHWEST: number;
    QUADRANT_SOUTH: number;
};

/**
 * Converts a color from HSV format to RGBA format.
 *
 * Based on: https://stackoverflow.com/a/17243070/2051724
 *
 * @param h - Hue (0.0 - 1.0).
 * @param s - Saturation (0.0 - 1.0).
 * @param v - Value (0.0 - 1.0).
 * @param a - Optional alpha (0.0 - 1.0).
 * @returns A 32-bit unsigned integer color.
 */
export declare function fromHsv(h: number, s: number, v: number, a?: number): Color;

/**
 * Creates a big-endian 32-bit RGBA color from red, green, and blue components.
 * @param r - Red (0-255).
 * @param g - Green (0-255).
 * @param b - Blue (0-255).
 * @param a - Optional alpha (0-255).
 * @returns A 32-bit unsigned integer color.
 */
export declare function fromRgb(r: number, g: number, b: number, a?: number): Color;

export declare function getFovQuadrant(dx: number, dy: number): number;

export declare class GUI {
    readonly terminal: Terminal;
    readonly renderer: DialogRenderer;
    readonly dialogs: DialogState[];
    constructor(terminal: Terminal, renderer?: DialogRenderer, dialogs?: DialogState[]);
    add(dialog: Dialog): void;
    /**
     * Handles input for currently active dialog.
     * Returns true if the input was handled.
     * Returns false otherwise.
     */
    handleInput(): boolean;
    draw(): void;
}

/**
 * The Input class represents a physical input.
 * Example: keyboard key or mouse button.
 */
export declare class Input {
    down: boolean;
    downTime: number;
    repeat: boolean;
    repeatTime: number;
    downCount: number;
    upCount: number;
    setDown(down: boolean): void;
    update(time: number): void;
    /**
     * Returns true if the input is "pressed".
     * Pressed is a one time event when the input first goes down.
     * It then repeats on repeat delay.
     */
    isPressed(): boolean;
    /**
     * Returns true if the input is "clicked".
     * Clicked is a one time event when the input first goes up.
     */
    isClicked(): boolean;
}

export declare class InputSet<T> {
    readonly inputs: Map<T, Input>;
    clear(): void;
    get(key: T): Input;
    updateAll(time: number): void;
}

export declare type Key = keyof typeof Keys;

export declare class Keyboard {
    readonly keys: InputSet<KeyCode>;
    /**
     * Creates a new keyboard module.
     *
     * @param el - DOM el to attach listeners.
     */
    constructor(el: HTMLElement);
    clear(): void;
    getKey(key: KeyCode): Input;
    setKey(e: KeyboardEvent, state: boolean): void;
    updateKeys(time: number): void;
}

export declare type KeyCode = (typeof Keys)[Key];

export declare const Keys: {
    readonly VK_BACKSPACE: "Backspace";
    readonly VK_TAB: "Tab";
    readonly VK_ENTER: "Enter";
    readonly VK_SHIFT_LEFT: "ShiftLeft";
    readonly VK_SHIFT_RIGHT: "ShiftRight";
    readonly VK_CONTROL_LEFT: "ControlLeft";
    readonly VK_CONTROL_RIGHT: "ControlRight";
    readonly VK_ALT_LEFT: "AltLeft";
    readonly VK_ALT_RIGHT: "AltRight";
    readonly VK_PAUSE: "Pause";
    readonly VK_CAPS_LOCK: "CapsLock";
    readonly VK_ESCAPE: "Escape";
    readonly VK_SPACE: "Space";
    readonly VK_PAGE_UP: "PageUp";
    readonly VK_PAGE_DOWN: "PageDown";
    readonly VK_END: "End";
    readonly VK_HOME: "Home";
    readonly VK_LEFT: "ArrowLeft";
    readonly VK_UP: "ArrowUp";
    readonly VK_RIGHT: "ArrowRight";
    readonly VK_DOWN: "ArrowDown";
    readonly VK_INSERT: "Insert";
    readonly VK_DELETE: "Delete";
    readonly VK_0: "Digit0";
    readonly VK_1: "Digit1";
    readonly VK_2: "Digit2";
    readonly VK_3: "Digit3";
    readonly VK_4: "Digit4";
    readonly VK_5: "Digit5";
    readonly VK_6: "Digit6";
    readonly VK_7: "Digit7";
    readonly VK_8: "Digit8";
    readonly VK_9: "Digit9";
    readonly VK_SEMICOLON: "Semicolon";
    readonly VK_EQUALS: "Equal";
    readonly VK_A: "KeyA";
    readonly VK_B: "KeyB";
    readonly VK_C: "KeyC";
    readonly VK_D: "KeyD";
    readonly VK_E: "KeyE";
    readonly VK_F: "KeyF";
    readonly VK_G: "KeyG";
    readonly VK_H: "KeyH";
    readonly VK_I: "KeyI";
    readonly VK_J: "KeyJ";
    readonly VK_K: "KeyK";
    readonly VK_L: "KeyL";
    readonly VK_M: "KeyM";
    readonly VK_N: "KeyN";
    readonly VK_O: "KeyO";
    readonly VK_P: "KeyP";
    readonly VK_Q: "KeyQ";
    readonly VK_R: "KeyR";
    readonly VK_S: "KeyS";
    readonly VK_T: "KeyT";
    readonly VK_U: "KeyU";
    readonly VK_V: "KeyV";
    readonly VK_W: "KeyW";
    readonly VK_X: "KeyX";
    readonly VK_Y: "KeyY";
    readonly VK_Z: "KeyZ";
    readonly VK_CONTEXT_MENU: "ContextMenu";
    readonly VK_NUMPAD0: "Numpad0";
    readonly VK_NUMPAD1: "Numpad1";
    readonly VK_NUMPAD2: "Numpad2";
    readonly VK_NUMPAD3: "Numpad3";
    readonly VK_NUMPAD4: "Numpad4";
    readonly VK_NUMPAD5: "Numpad5";
    readonly VK_NUMPAD6: "Numpad6";
    readonly VK_NUMPAD7: "Numpad7";
    readonly VK_NUMPAD8: "Numpad8";
    readonly VK_NUMPAD9: "Numpad9";
    readonly VK_NUMPAD_ENTER: "NumpadEnter";
    readonly VK_MULTIPLY: "NumpadMultiply";
    readonly VK_ADD: "NumpadAdd";
    readonly VK_SEPARATOR: "NumpadSeparator";
    readonly VK_SUBTRACT: "NumpadSubtract";
    readonly VK_DECIMAL: "NumpadDecimal";
    readonly VK_DIVIDE: "NumpadDivide";
    readonly VK_F1: "F1";
    readonly VK_F2: "F2";
    readonly VK_F3: "F3";
    readonly VK_F4: "F4";
    readonly VK_F5: "F5";
    readonly VK_F6: "F6";
    readonly VK_F7: "F7";
    readonly VK_F8: "F8";
    readonly VK_F9: "F9";
    readonly VK_F10: "F10";
    readonly VK_F11: "F11";
    readonly VK_F12: "F12";
    readonly VK_F13: "F13";
    readonly VK_F14: "F14";
    readonly VK_F15: "F15";
    readonly VK_F16: "F16";
    readonly VK_F17: "F17";
    readonly VK_F18: "F18";
    readonly VK_F19: "F19";
    readonly VK_F20: "F20";
    readonly VK_F21: "F21";
    readonly VK_F22: "F22";
    readonly VK_F23: "F23";
    readonly VK_F24: "F24";
    readonly VK_NUM_LOCK: "NumLock";
    readonly VK_SCROLL_LOCK: "ScrollLock";
    readonly VK_COMMA: "Comma";
    readonly VK_PERIOD: "Period";
    readonly VK_SLASH: "Slash";
    readonly VK_BACKQUOTE: "Backquote";
    readonly VK_OPEN_BRACKET: "BracketLeft";
    readonly VK_BACK_SLASH: "Backslash";
    readonly VK_CLOSE_BRACKET: "BracketRight";
    readonly VK_QUOTE: "Quote";
    readonly VK_META: "OSLeft";
};

export declare function loadImage(url: string, callback: (img: Console_2) => void): void;

export declare function loadImage2x(url: string, callback: (img: Console_2) => void): void;

export declare class Message {
    readonly text: string | undefined;
    readonly fg?: number | undefined;
    readonly bg?: number | undefined;
    readonly children?: Message[] | undefined;
    readonly align?: string | undefined;
    constructor(text: string | undefined, fg?: number | undefined, bg?: number | undefined, children?: Message[] | undefined, align?: string | undefined);
    getWidth(): number;
    getHeight(): number;
}

export declare const MessageAlign: {
    LEFT: string;
    CENTER: string;
    RIGHT: string;
};

export declare class MessageDialog extends Dialog {
    readonly message: string | Message;
    constructor(title: string, message: string | Message);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal): boolean;
}

export declare class Mouse {
    readonly el: HTMLCanvasElement;
    readonly width: number;
    readonly height: number;
    readonly buttons: InputSet<number>;
    private prevX;
    private prevY;
    x: number;
    y: number;
    dx: number;
    dy: number;
    wheelDeltaX: number;
    wheelDeltaY: number;
    constructor(terminal: Terminal);
    private handleTouchEvent;
    private handleEvent;
    private handleWheelEvent;
    private updatePosition;
    update(time: number): void;
}

/**
 * PICO 8 palette.
 * See: https://pico-8.fandom.com/wiki/Palette
 * See: https://lospec.com/palette-list/pico-8
 */
export declare const Pico8Palette: {
    BLACK: number;
    DARK_BLUE: number;
    DARK_PURPLE: number;
    DARK_GREEN: number;
    BROWN: number;
    DARK_GRAY: number;
    LIGHT_GRAY: number;
    WHITE: number;
    RED: number;
    ORANGE: number;
    YELLOW: number;
    GREEN: number;
    BLUE: number;
    LAVENDER: number;
    PINK: number;
    LIGHT_PEACH: number;
};

export declare class Point implements PointLike {
    readonly x: number;
    readonly y: number;
    constructor(x: number, y: number);
}

export declare interface PointLike {
    readonly x: number;
    readonly y: number;
}

export declare class Rect {
    readonly x: number;
    readonly y: number;
    readonly width: number;
    readonly height: number;
    constructor(x: number, y: number, width: number, height: number);
    get left(): number;
    get top(): number;
    get x2(): number;
    get y2(): number;
    getCenter(): Point;
    intersects(other: Rect): boolean;
    contains(point: Point | Rect): boolean;
}

/**
 * Random number generator.
 *
 * LCG
 * https://stackoverflow.com/a/424445/2051724
 */
export declare class RNG {
    private m;
    private a;
    private c;
    private state;
    /**
     * Creates a new random number generator.
     *
     * @param seed - The integer seed.
     */
    constructor(seed?: number);
    nextInt(): number;
    /**
     * Returns a floating point number between 0.0 and 1.0.
     */
    nextFloat(): number;
    /**
     * Returns an integer in the range start (inclusive) to end (exclusive).
     * @param start - Lower bound, inclusive.
     * @param end - Upper bound, exclusive.
     */
    nextRange(start: number, end: number): number;
    chooseIndex(chances: number[]): number;
    chooseKey(chancesMap: Record<string, number>): string;
}

export declare class ScrollableMessageDialog extends Dialog {
    readonly message: Message;
    readonly messagesHeight: number;
    readonly scrollMax: number;
    readonly scrollbarHeight: number;
    scrollY: number;
    constructor(rect: Rect, title: string, message: Message);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal): boolean;
}

export declare class SelectDialog extends Dialog {
    readonly options: string[];
    readonly callback: (i: number) => void;
    private hoverIndex;
    constructor(title: string, options: string[], callback: (i: number) => void);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal, offset: Point): boolean;
    isMouseOverOption(terminal: Terminal, offset: Point, index: number): boolean;
}

/**
 * Decorates a class to make serializable.
 * Any class with the `@serializable` decorator will be serialized and deserialized.
 * @param value - The TypeScript class to mark as serializable.
 */
export declare function serializable(value: {
    name: string;
}): void;

/**
 * Serializes a value to JSON.
 * Handles circular references and class instances.
 * @param obj - The root object to serialize.
 * @returns A string representation of the object graph.
 */
export declare function serialize(obj: unknown): string;

export declare class Terminal extends Console_2 {
    readonly canvas: HTMLCanvasElement;
    readonly font: Font;
    readonly crt?: CrtOptions;
    readonly maxFps?: number;
    readonly pixelWidth: number;
    readonly pixelHeight: number;
    readonly pixelScale: number;
    readonly keys: Keyboard;
    readonly mouse: Mouse;
    readonly gl: WebGLRenderingContext;
    readonly program: WebGLProgram;
    readonly positionAttribLocation: number;
    readonly textureAttribLocation: number;
    readonly fgColorAttribLocation: number;
    readonly bgColorAttribLocation: number;
    readonly positionsArray: Float32Array;
    readonly indexArray: Uint16Array;
    readonly textureArray: Float32Array;
    readonly foregroundUint8Array: Uint8Array;
    readonly foregroundDataView: DataView;
    readonly backgroundUint8Array: Uint8Array;
    readonly backgroundDataView: DataView;
    readonly positionBuffer: WebGLBuffer;
    readonly indexBuffer: WebGLBuffer;
    readonly textureBuffer: WebGLBuffer;
    readonly foregroundBuffer: WebGLBuffer;
    readonly backgroundBuffer: WebGLBuffer;
    readonly texture: WebGLTexture;
    readonly frameBufferTexture: WebGLTexture;
    readonly frameBuffer: WebGLFramebuffer;
    readonly crtProgram: WebGLProgram;
    readonly crtBlurLocation: WebGLUniformLocation;
    readonly crtCurvatureLocation: WebGLUniformLocation;
    readonly crtChromaLocation: WebGLUniformLocation;
    readonly crtScanlineWidthLocation: WebGLUniformLocation;
    readonly crtScanlineIntensityLocation: WebGLUniformLocation;
    readonly crtVignetteLocation: WebGLUniformLocation;
    readonly crtPositionLocation: number;
    readonly crtTexCoordLocation: number;
    readonly crtPositionBuffer: WebGLBuffer;
    readonly crtTexCoordBuffer: WebGLBuffer;
    private lastRenderTime;
    private renderDelta;
    fps: number;
    averageFps: number;
    update?: () => void;
    constructor(canvas: HTMLCanvasElement, width: number, height: number, options?: TerminalOptions);
    private handleResize;
    private getAttribLocation;
    private flush;
    isKeyDown(key: KeyCode): boolean;
    isKeyPressed(key: KeyCode): boolean;
    getKeyDownCount(key: KeyCode): number;
    /**
     * Returns a standard roguelike movement key if pressed.
     * Implemented control systems:
     * 1) Numpad arrows
     * 2) VIM keys
     * 3) Normal arrows (4 directions only)
     * 4) Numpad 5 and '.' (period) for "wait"
     * If a key is pressed, returns the movement delta.
     * If no key is pressed, returns undefined.
     * See: http://www.roguebasin.com/index.php?title=Preferred_Key_Controls
     */
    getMovementKey(movementKeys?: Partial<Record<KeyCode, Point>>): Point | undefined;
    private buildShader;
    /**
     * Initialize a texture and load an image.
     * When the image finished loading copy it into the texture.
     * @param url - The texture image URL.
     */
    private loadTexture;
    private render;
    private renderCrt;
    private requestAnimationFrame;
    private renderLoop;
}

export declare interface TerminalOptions {
    font?: Font;
    movementKeys?: Partial<Record<KeyCode, Point>>;
    crt?: CrtOptions;
    maxFps?: number;
}

/**
 * Returns the input string wrapped to the maximum line length.
 * @param str - The original input string.
 * @param maxLength - The maximum length of a single line.
 * @returns Array of word wrapped lines.
 */
export declare function wordWrap(str: string, maxLength: number): string[];

export { }
